@@ -20,9 +20,9 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 
-#define INDICATOR_APPLICATION_DBUS_ADDR        "org.ayatana.indicator.application"
-#define INDICATOR_APPLICATION_DBUS_OBJ         "/org/ayatana/indicator/application/service"
-#define INDICATOR_APPLICATION_DBUS_IFACE       "org.ayatana.indicator.application.service"
+#define INDICATOR_APPLICATION_DBUS_ADDR        "com.canonical.indicator.application"
+#define INDICATOR_APPLICATION_DBUS_OBJ         "/com/canonical/indicator/application/service"
+#define INDICATOR_APPLICATION_DBUS_IFACE       "com.canonical.indicator.application.service"
 
 #define NOTIFICATION_WATCHER_DBUS_ADDR    "org.kde.StatusNotifierWatcher"
 #define NOTIFICATION_WATCHER_DBUS_OBJ     "/StatusNotifierWatcher"
@@ -31,5 +31,5 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define NOTIFICATION_ITEM_DBUS_IFACE      "org.kde.StatusNotifierItem"
 #define NOTIFICATION_ITEM_DEFAULT_OBJ     "/StatusNotifierItem"
 
-#define NOTIFICATION_APPROVER_DBUS_IFACE  "org.ayatana.StatusNotifierApprover"
+#define NOTIFICATION_APPROVER_DBUS_IFACE  "com.canonical.StatusNotifierApprover"
 
